@@ -3,17 +3,17 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 use JSON;
 use Carp;
 use Scalar::Util 'reftype';
 use List::Util;
 use Pandoc::Walker;
-use Pandoc::Elements ();
+use Pandoc::Elements qw(Image Str);
 
 use parent 'Exporter';
-our @EXPORT = qw(pandoc_filter pandoc_walk stringify);
+our @EXPORT = qw(pandoc_filter pandoc_walk build_image stringify);
 
 # FUNCTIONS
 
@@ -39,6 +39,25 @@ sub pandoc_filter(@) {    ## no critic
     say STDOUT $json;
 }
 
+sub build_image {
+    my $e = shift;
+    my $filename = shift // '';
+
+    my $img = Image [$e->id, $e->classes, []], [], [$filename, ''];
+    my $keyvals = $e->keyvals;
+
+    my $caption = $keyvals->get('caption');
+    if (defined $caption) {
+        push @{$img->content}, Str($caption);
+        $img->target->[1] = 'fig:';
+        $keyvals->remove('caption');
+    }
+    $img->keyvals($keyvals);
+
+    return $img;
+}
+
+
 # METHODS
 
 sub new {
@@ -183,6 +202,14 @@ AST as single line of JSON. This function is roughly equivalent to
     Pandoc::Filter->new(@actions)->apply($ast, $format);
     say $ast->to_json;
 
+=head2 build_image( $element [, $filename ] )
+
+Maps an element to an L<Image|Pandoc::Elements/Image> element with attributes
+from the given element. The attribute C<caption>, if available, is transformed
+into image caption. This utility function is useful for filters that transform
+content to images. See graphviz, tikz, lilypond and similar filters in the
+L<examples|https://metacpan.org/pod/distribution/Pandoc-Elements/examples/>.
+
 =head1 SEE ALSO
 
 Script L<pandocwalk>, installed with this module, facilitates execution of
